/*
 * Copyright (C) 2012 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
//`define USE_FIFO
`define USE_SPI
//`define USE_BUS

//  XO2-256         UDRV                XO2-256
//  27           1            24  VCC
//  28           2            23  nBUSY       25
//  29  nRD      3(TMS)       22              23
//  30  nWR      4(TCK)       21  CLK_OUT     21
//  32  DB0      5(TDI)       20  CLK_4X      20
//   1  DB1      6(TDO)       19  nDATA_FULL  17
//   4  DB2      7            18  nDATA_EMPTY 16
//   5  DB3      8            17  N.C. (JTAGENB)
//   8  DB4      9            16  nDATA_RDY   14
//   9  DB5      10           15  nCS         13
//  10  DB6      11           14  RS          12
//       GND     12           13  DB7         11
//
//  XO2-256        UDRV SPI             XO2-256
//  27           1            24  VCC
//  28           2            23  nBUSY       25
//  29  nCS(2)   3(TMS)       22              23
//  30  SCK(2)   4(TCK)       21  CLK_OUT     21
//  32  MOSI(2)  5(TDI)       20  CLK_4X      20
//   1  MISO(2)  6(TDO)       19  nDATA_FULL  17
//   4           7            18  nDATA_EMPTY 16
//   5           8            17  N.C. (JTAGENB)
//   8           9            16  nDATA_RDY   14
//   9  MISO     10           15  nCS         13
//  10  MOSI     11           14  RS          12
//       GND     12           13  SCK         11



module dmy_top # (
		parameter FS=1
  		, parameter FIFO_DEPTH = 5
//  		, parameter FIFO_DEPTH = 6
	) (
                input CLK_4X
`ifdef USE_SPI
		, input MOSI
		, input SCK
		, input nCS
		, output MISO
                , input RS
`elsif USE_BUS
                , inout [7:0] DB
                , input nWR
                , input nRD
                , input RS
                , input nCS
`endif
	);

    udrv_fifo # ( .DEPTH(FIFO_DEPTH)
		  , .DELAYED_INPUT(1)
	)
	fifo_impl (.CLK(CLK_4X), .RST(FIFO_RST)
		, .I_DATA(I_DATA[7:0]), .O_DATA(O_DATA[7:0])
		, .I_DATA_RDY(I_DATA_RDY) , .I_DATA_ACK(I_DATA_ACK)
		, .O_DATA_RDY(O_DATA_RDY) , .O_DATA_ACK(O_DATA_ACK)
		, .I_DATA_FULL(I_DATA_FULL)
		, .I_DATA_EMPTY(I_DATA_EMPTY)
	);
`ifdef USE_SPI
    wire [7:0] SPI_DATA_I;
    udrv_spi spi_impl (.MOSI(MOSI), .SCK(SCK), .nCS(nCS), .MISO(MISO)
		, .SPI_DATA_I(SPI_DATA_I[7:0])
		, .SPI_DATA_I_ACK(SPI_DATA_I_ACK)
		, .SPI_DATA_I_RDY(SPI_DATA_I_RDY)

		, .SPI_DATA(SEND_DATA[7:0])
		, .SPI_DATA_RDY(SPI_DATA_RDY)
		, .SPI_DATA_ACK(SPI_DATA_ACK)
	);

    assign SPI_DATA_I[7:0] =  (~RS & r_recv_mode ) ? RECV_DATA[7:0]
						   : CTRL_REG[7:0] ;

    reg dummy_i_rdy;
    reg dummy_o_ack;
    always @(posedge CLK)
    begin
    	dummy_o_ack <= SPI_DATA_RDY;
    	dummy_i_rdy <= ~SPI_DATA_I_ACK;
	if (RS & SPI_DATA_RDY & SEND_DATA[7])
	  begin
	    r_recv_mode  <= SEND_DATA[1];
	    r_start      <= SEND_DATA[0];
	  end
	else
	  begin
	    if (SE0)
		if (~r_recv_mode) r_recv_mode <= 1;
		else r_start <= 1'b0;
		//r_start <= 1'b0;
	  end
    end
    assign RECV_DATA_ACK = ~RS & SPI_DATA_I_ACK;
    //assign SPI_DATA_I_RDY = ~RS ? RECV_DATA_RDY : dummy_i_rdy;
    assign SPI_DATA_I_RDY = dummy_i_rdy;
    assign SEND_DATA_RDY = ~RS & SPI_DATA_RDY;
    assign SPI_DATA_ACK = ~RS & ~i_recv_mode ? SEND_DATA_ACK : dummy_o_ack;

`elsif USE_BUS
    assign DB[7:0] =  (~nRD & ~nCS & ~RS) ? RECV_DATA[7:0]
		    : (~nRD & ~nCS &  RS) ? CTRL_REG[7:0]
		    : 8'bz;
    assign SEND_DATA[7:0] = DB[7:0];

    always @(posedge CLK)
    begin
	if (RS & ~(nWR | nCS))
	  begin
	    r_recv_mode <= DB[1];
	    r_start     <= DB[0];
	  end
	else
	  begin
	    if (SE0)
		if (~r_recv_mode) r_recv_mode <= 1;
		else r_start <= 1'b0;
	        //r_start <= 1'b0;
	  end
    end
    assign RECV_DATA_ACK = ~(nRD | nCS);
    assign SEND_DATA_RDY = ~(nWR | nCS) & ~RS;
//
// BUS Hand shake 
//
//       _  ____________________________________  _____
// RS    _><____________________________________><_____
//
//       _____                                  _______
// nCS        |________________________________|
//
//        ______________               _____________
// nWR/nRD              |_____________|
//  
//                         _______________
// DB[7:0]  --------------<_______________>----------
//  
`endif

endmodule

`ifdef USE_FIFO
//
// Hand shake 
//         ______________________  _______
// DATA   <______________________><_______
//
//                _______                _____
// RDY   ________|       |______________|
//  
//                    ________
// ACK  _____________|       |________________
//  
//
// CLK   |       |       |       |       |  
//  
module udrv_fifo # (
//  		parameter DEPTH = 5
		parameter DEPTH = 6
		, parameter DELAYED_INPUT = 0
	) (
                input CLK
                , input RST
                , input [7:0] I_DATA
                , input I_DATA_RDY
                , output I_DATA_ACK
                , output I_DATA_FULL
                , output I_DATA_EMPTY
                , output [7:0] O_DATA
                , output O_DATA_RDY
                , input O_DATA_ACK
	);
    reg  [7:0] mem [0:2**DEPTH-1];

    reg [DEPTH-1:0] i_addr = 0;
    reg [DEPTH-1:0] o_addr = 0;
    wire [DEPTH-1:0] i_next = i_addr + 1;
    wire [DEPTH-1:0] o_next = o_addr + 1;
    reg i_rdy = 0;
    reg i_ack = 0;
    reg o_rdy = 0;
    reg o_ack = 0;
    reg r_full = 0;
    assign I_DATA_FULL = r_full;
    assign I_DATA_EMPTY = (i_addr == o_addr);
    assign I_DATA_ACK = i_ack;
    assign O_DATA_RDY = o_rdy;
    assign DATA_RDY = DELAYED_INPUT ? i_rdy : I_DATA_RDY;

    always @(negedge CLK)
    begin
        i_rdy <= I_DATA_RDY;
        o_ack <= O_DATA_ACK;
        if (RST)
	  begin
	    i_addr <= 0;
	    o_addr <= 0;
            i_ack <= 1'b0;
            o_rdy <= 1'b0;
            r_full <= 1'b0;
	  end
        else if (DATA_RDY & ~i_ack) 
	  begin
	    if (i_next == o_addr)
	      r_full <= 1'b1;
	    else
	      begin
		mem[i_addr] <= I_DATA;
		i_addr <= i_next;
	      end
            i_ack <= 1'b1;
	  end
	else
	    i_ack <= DATA_RDY;

        if ( o_ack & ~O_DATA_ACK & (i_addr != o_addr))
	  begin
	    o_addr <= o_next;
            o_rdy <= 1'b0;
	  end
        else if ( O_DATA_ACK )
            o_rdy <= 1'b0;
        else
	    o_rdy <= (i_addr != o_addr);
    end
    assign O_DATA[7:0] = mem[o_addr];

endmodule
`endif

`ifdef USE_SPI
module udrv_spi (
		  input MOSI
		, input SCK
		, input nCS
		, output MISO

		, input [7:0] SPI_DATA_I
		, output SPI_DATA_I_ACK
		, input SPI_DATA_I_RDY

		, output [7:0] SPI_DATA
		, output SPI_DATA_RDY
		, input SPI_DATA_ACK
	);

    reg [2:0]  r_count = 0;
    reg [7:0]  r_spi = 0;
    reg [7:0]  spi_out = 0;
    reg r_mosi = 1'b0;

    reg o_data_ack  = 1'b0;
    reg o_data_rdy  = 1'b0;
    assign SPI_DATA_RDY   = o_data_rdy;

    reg i_data_ack  = 1'b0;
    assign SPI_DATA_I_ACK = i_data_ack;    
     
    assign SPI_DATA       = spi_out[7:0];
    assign MISO = (r_count == 0) ? SPI_DATA_I[7] : r_spi[7];

    always @(posedge SCK)
    begin
	r_mosi <= MOSI;
	if (r_count == 1)
	    i_data_ack <= 1'b1;
	else if (~SPI_DATA_I_RDY)
     	    i_data_ack <= 1'b0;
    end

    always @(posedge nCS or negedge SCK)
    begin
	if (nCS) //  LOAD
	  begin
	    r_count <= 0;
	    r_spi <= 0;
	    o_data_rdy <= 1'b0;
	    o_data_ack  <= 1'b0;
	  end
	else
	  begin
	    r_count <= r_count + 1;
	    o_data_ack  <=  SPI_DATA_ACK;
	    if (r_count == 0)
		r_spi <=  { SPI_DATA_I[6:0], r_mosi };	//  LOAD
	    else
	        r_spi <= {r_spi[6:0], r_mosi };		// SHIFT

	    if (r_count == 7)
		spi_out <= {r_spi[6:0], r_mosi };

	    if (r_count == 7)
	        o_data_rdy <= 1'b1;
	    else if (~o_data_ack & SPI_DATA_ACK)
	        o_data_rdy <= 1'b0;
	  end
    end
endmodule
`endif
