
module dac_top (
		, input MOSI
		, input SCK
		, input nCS
		, output MISO
	);

    wire [7:0] SPI_DATA_I;
    udrv_spi spi_impl (.MOSI(MOSI), .SCK(SCK), .nCS(nCS), .MISO(MISO)
		, .SPI_DATA_I(SPI_DATA_I[7:0])
		, .SPI_DATA_I_ACK(SPI_DATA_I_ACK)
		, .SPI_DATA_I_RDY(SPI_DATA_I_RDY)

		, .SPI_DATA(SEND_DATA[7:0])
		, .SPI_DATA_RDY(SPI_DATA_RDY)
		, .SPI_DATA_ACK(SPI_DATA_ACK)
	);

    assign SPI_DATA_I[7:0] =  (RS & r_recv_mode ) ? RECV_DATA[7:0]
						   : CTRL_REG[7:0] ;

    reg dummy_i_rdy;
    reg dummy_o_ack;
    always @(posedge CLK)
    begin
	r_recv_mode1 <= i_recv_mode;
    	dummy_o_ack <= SPI_DATA_RDY;
    	dummy_i_rdy <= ~SPI_DATA_I_ACK;
	if (~RS & SPI_DATA_RDY & SEND_DATA[7])
	  begin
	    r_recv_mode  <= SEND_DATA[1];
	    r_start      <= SEND_DATA[0];
	  end
	else
	  begin
	    if (SE0)
	      begin
`ifdef AUTO_RECV_AFTER_SEND
		if (~r_recv_mode) r_recv_mode <= 1;
`endif
		r_start <= 1'b0;
	      end
	  end
    end
    assign RECV_DATA_ACK = RS & SPI_DATA_I_ACK;
    //assign SPI_DATA_I_RDY = RS ? RECV_DATA_RDY : dummy_i_rdy;
    assign SPI_DATA_I_RDY = dummy_i_rdy;
    assign SEND_DATA_RDY = RS & SPI_DATA_RDY;
    assign SPI_DATA_ACK = RS & ~i_recv_mode ? SEND_DATA_ACK : dummy_o_ack;

endmodule

module udrv_spi (
		  input MOSI
		, input SCK
		, input nCS
		, output MISO

		, input [7:0] SPI_DATA_I
		, output SPI_DATA_I_ACK
		, input SPI_DATA_I_RDY

		, output [7:0] SPI_DATA
		, output SPI_DATA_RDY
		, input SPI_DATA_ACK
		, output [2:0] SPI_INDEX
	);

    reg [2:0]  r_count = 0;
    reg [7:0]  r_spi = 0;
    reg [7:0]  spi_out = 0;
    reg r_mosi = 1'b0;
    reg [2:0]  data_bytes = 0;

    reg o_data_ack  = 1'b0;
    reg o_data_rdy  = 1'b0;
    assign SPI_DATA_RDY   = o_data_rdy;

    reg i_data_ack  = 1'b0;
    assign SPI_DATA_I_ACK = i_data_ack;    
     
    assign SPI_DATA       = spi_out[7:0];
    assign MISO = (r_count == 0) ? SPI_DATA_I[7] : r_spi[7];

    always @(posedge SCK)
    begin
	r_mosi <= MOSI;
	if (r_count == 1)
	    i_data_ack <= 1'b1;
	else if (~SPI_DATA_I_RDY)
     	    i_data_ack <= 1'b0;
    end

    always @(posedge nCS or negedge SCK)
    begin
	if (nCS) //  LOAD
	  begin
	    r_count <= 0;
	    r_spi <= 0;
	    o_data_rdy <= 1'b0;
	    o_data_ack  <= 1'b0;
	    data_bytes <= 0;
	  end
	else
	  begin
	    r_count <= r_count + 1;
	    o_data_ack  <=  SPI_DATA_ACK;
	    if (r_count == 0)
		r_spi <=  { SPI_DATA_I[6:0], r_mosi };	//  LOAD
	    else
	        r_spi <= {r_spi[6:0], r_mosi };		// SHIFT

	    if (r_count == 7)
		spi_out <= {r_spi[6:0], r_mosi };

	    if (r_count == 7)
	        o_data_rdy <= 1'b1;
	    else if (~o_data_ack & SPI_DATA_ACK)
	        o_data_rdy <= 1'b0;

	    if ( (r_count == 7) & (&data_bytes != 1'b1) )
	        data_bytes <= data_bytes + 1;
	  end
    end
endmodule
