/*
 * Copyright (C) 2012 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
`define USE_TOP
`ifdef USE_TOP
  `define USE_CLKGEN
  `define USE_RECIEVER
  `define USE_SENDER
  `define USE_FIFO
  `define USE_BUS
`endif
//`define USE_CLKGEN
//`define USE_RECIEVER
//`define USE_SENDER
//`define USE_FIFO
`define USE_CRC
`define USE_SPI
//`define USE_BUS

//`define LB_TEST

`define ALWAYS_RECV_START	// must
`define AUTO_RECV_AFTER_SEND
`define USE_LATCHED_DP
`define USE_DUAL_CRC5
`define SUPPORT_CLK_OUT

`ifdef LB_TEST
`else
  `define USE_DCMA
`endif
  
`ifdef USE_TOP

//  XO2-256         UDRV                XO2-256
//  27  DP       1            24  VCC
//  28  DM       2            23  nBUSY       25
//  29  nRD      3(TMS)       22  nCRC_ERROR  23
//  30  nWR      4(TCK)       21  CLK_OUT     21
//  32  DB0      5(TDI)       20  CLK_4X      20
//   1  DB1      6(TDO)       19  nDATA_FULL  17
//   4  DB2      7            18  nDATA_EMPTY 16
//   5  DB3      8            17  N.C. (JTAGENB)
//   8  DB4      9            16  nDATA_RDY   14
//   9  DB5      10           15  nCS         13
//  10  DB6      11           14  RS          12
//       GND     12           13  DB7         11
//
//  XO2-256        UDRV SPI             XO2-256
//  27  DP       1            24  VCC
//  28  DM       2            23  nBUSY       25
//  29  nCS(2)   3(TMS)       22  nCRC_ERROR  23
//  30  SCK(2)   4(TCK)       21  CLK_OUT     21
//  32  MOSI(2)  5(TDI)       20  CLK_4X      20
//   1  MISO(2)  6(TDO)       19  nDATA_FULL  17
//   4           7            18  nDATA_EMPTY 16
//   5           8            17  N.C. (JTAGENB)
//   8           9            16  nDATA_RDY   14
//   9  MISO     10           15  nCS         13
//  10  MOSI     11           14  RS          12
//       GND     12           13  SCK         11

module udrv # (
		parameter SUPPORT_CRC5S=1
		, parameter FS=1
  		, parameter FIFO_DEPTH = 5
//  		, parameter FIFO_DEPTH = 6
	) (
                input CLK_4X
`ifdef SUPPORT_CLK_OUT
                , output CLK_OUT
`endif
                , output nBUSY

`ifdef LB_TEST
                , input DPR
                , input DMR
                , output DPW
                , output DMW
`else
                , inout DP
                , inout DM
`endif

                , output nDATA_FULL
                , output nDATA_EMPTY
                , output nDATA_RDY
`ifdef USE_SPI
		, input MOSI
		, input SCK
		, input nCS
		, output MISO
                , input RS
`elsif USE_BUS
                , inout [7:0] DB
                , input nWR
                , input nRD
                , input RS
                , input nCS
`else
                , input RECV_MODE
                , input SEND_START
                , output [7:0] RECV_DATA
                , input [7:0] SEND_DATA

                , input RECV_DATA_ACK

                , input SEND_DATA_RDY
                , output SEND_DATA_ACK
`endif
`ifdef USE_CRC
                , output nCRC_ERROR
`endif
	);

`ifdef SUPPORT_CLK_OUT
    wire CLK_OUT = ~CLK_4X;
`endif
`ifdef USE_DCMA
  DCMA clk_selector (.CLK0(CLK_4X), .CLK1(1'b0), .DCMOUT(i_clk4x)
        , .SEL(1'b0) // 0: CLK0 1: CLK1
        );
`else
    wire i_clk4x = CLK_4X;
`endif

`ifdef USE_CRC
   wire crc5_in;
   wire crc5_out;
   wire [1:0] crc5_sel;

   wire crc16_in;
   wire crc16_out;
   wire [1:0] crc16_sel;

   wire nCRC_ERROR;
   wire crc_in;
   wire [1:0] r_crc5_sel;
   wire [1:0] r_crc16_sel;

   wire s_crc5_in;
   wire s_crc16_in;
   wire s_crc_out;
   wire [1:0] s_crc5_sel;
   wire [1:0] s_crc16_sel;
`endif
`ifdef USE_SPI
   `define HAVE_CTRL_REG
`elsif USE_BUS
   `define HAVE_CTRL_REG
`endif

`ifdef HAVE_CTRL_REG
    wire [7:0] RECV_DATA;
    wire [7:0] SEND_DATA;

    wire RECV_DATA_ACK;

    wire SEND_DATA_RDY;
    wire SEND_DATA_ACK;
`endif

    wire DPW;
    wire DPM;
    wire DOE;
`ifdef LB_TEST
    wire DPR;
    wire DMR;
    wire SE0 = (~DPR & ~DMR) | (~DPW & ~DMW);
`else
    assign DP = DOE ? DPW : 1'bz;
    assign DM = DOE ? DMW : 1'bz;
    wire DPR = DP;
    wire SE0 = (~DP & ~DM);
`endif
`ifdef USE_LATCHED_DP
    reg r_dpr = 0;
    reg r_se0 = 0;

    always @(negedge CLK)
    begin
	r_dpr <= DPR;
	r_se0 <= SE0;
    end
`else
    wire r_dpr = DPR;
    wire r_se0 = SE0;
`endif
    wire i_recv_mode;
    wire i_send_start;
    wire i_recv_start;

    reg r_sync_req1 = 0;
    reg r_sync_req = 0;
    reg r_send_en = 0;
    reg r_recv_en1 = 0;
    reg r_recv_en = 0;
    reg r_recv_mode1 = 0;
    assign SYNC_REQ = (r_sync_req1 | r_sync_req);
    assign RECV_EN =  (r_recv_en1 | r_recv_en);
    wire   FIFO_RST = (r_recv_mode1 ^ i_recv_mode);
    assign SEND_EN = r_send_en;

    wire [7:0] S_DATA;
    wire [7:0] R_DATA;
    wire [7:0] I_DATA;
    wire [7:0] O_DATA;

    udrv_clkgen 
`ifdef USE_LATCHED_DP
		# ( .PH(2) )
`endif
	clkgen_impl (.CLK_4X(i_clk4x), .DP(DPR), .CLK(CLK)
                , .SYNC_REQ(SYNC_REQ), .SYNC_ACK(SYNC_ACK)
	);
    udrv_fifo # ( .DEPTH(FIFO_DEPTH)
`ifdef USE_SPI
		  , .DELAYED_INPUT(1)
`elsif USE_BUS
		  , .DELAYED_INPUT(1)
`endif
	)
	fifo_impl (
`ifdef USE_SPI
		.CLK(CLK)
`elsif USE_BUS
		.CLK(i_clk4x)
`else
		.CLK(CLK)
`endif
		, .RST(FIFO_RST)
		, .I_DATA(I_DATA[7:0]), .O_DATA(O_DATA[7:0])
		, .I_DATA_RDY(I_DATA_RDY) , .I_DATA_ACK(I_DATA_ACK)
		, .O_DATA_RDY(O_DATA_RDY) , .O_DATA_ACK(O_DATA_ACK)
		, .I_DATA_FULL(I_DATA_FULL)
		, .I_DATA_EMPTY(I_DATA_EMPTY)
	);
    udrv_reciever reciever_impl (.CLK(CLK), .RECV_EN(RECV_EN)
		, .DP(r_dpr), .SE0(r_se0)
		, .RECV_DATA(R_DATA[7:0])
		, .RECV_DATA_RDY(R_DATA_RDY), .RECV_DATA_ACK(R_DATA_ACK)
`ifdef USE_CRC
		, .SEL5(r_crc5_sel[1:0]), .SEL16(r_crc16_sel[1:0])
		, .O(crc_in), .MATCH_SEL(crc_match_sel)
`endif
	);
    udrv_sender # ( .FS(FS), .SUPPORT_CRC5S(SUPPORT_CRC5S) )
	sender_impl (.CLK(CLK), .SEND_EN(SEND_EN), .DP(DPW), .DM(DMW), .OE(DOE)
		, .SEND_DATA(S_DATA[7:0])
		, .SEND_DATA_RDY(S_DATA_RDY), .SEND_DATA_ACK(S_DATA_ACK)
                , .SEND_DONE(SEND_DONE)
`ifdef USE_CRC
		, .SEL5(s_crc5_sel[1:0]), .SEL16(s_crc16_sel[1:0])
		, .I5(s_crc5_in), .I16(s_crc16_in) , .O(s_crc_out)
`endif
	);

`ifdef USE_CRC
`ifdef USE_DUAL_CRC5
    udrv_crc #(.WIDTH(5), .SEQUENTIAL_MATCHING(1) )
	 crc5s_impl (.CLK(CLK), .SEL(s_crc5_sel[1:0]), .I(s_crc_out)
		, .O(crc5_out)
	); 
    udrv_crc #(.WIDTH(5), .SEQUENTIAL_MATCHING(1) )
	 crc5r_impl (.CLK(CLK), .SEL(r_crc5_sel[1:0]), .I(crc_in)
		, .MATCH(CRC5_MATCH)
	); 
`else
    udrv_crc #(.WIDTH(5), .SEQUENTIAL_MATCHING(1) )
	 crc5_impl (.CLK(CLK), .SEL(crc5_sel[1:0]), .I(crc5_in)
		, .O(crc5_out), .MATCH(CRC5_MATCH)
	); 
`endif
    udrv_crc #(.WIDTH(16), .SEQUENTIAL_MATCHING(0) )
	 crc16_impl (.CLK(CLK), .SEL(crc16_sel[1:0]), .I(crc16_in)
		, .O(crc16_out), .MATCH(CRC16_MATCH)
	); 
`endif
`ifdef HAVE_CTRL_REG
    reg r_recv_mode = 1'b0;
    reg r_start = 1'b0;
    assign nBUSY = r_start;

    wire [7:0]CTRL_REG = { 
   `ifdef USE_CRC
                ~nCRC_ERROR	// CRC ERROR
   `else
                1'b0
   `endif
                , I_DATA_FULL
		, I_DATA_EMPTY
		, ~nDATA_RDY
                , RECV_EN
                , SEND_EN
                , r_recv_mode
                , r_start
		};

    assign i_recv_mode  = r_recv_mode;
    assign i_send_start = ~i_recv_mode & r_start;
  `ifdef ALWAYS_RECV_START
    assign i_recv_start = i_recv_mode;
  `else
    assign i_recv_start = i_recv_mode & r_start;
  `endif
`else
    assign i_recv_mode  = RECV_MODE;
    assign i_send_start = SEND_START;
`endif

`ifdef USE_SPI
    wire [7:0] SPI_DATA_I;
    udrv_spi spi_impl (.MOSI(MOSI), .SCK(SCK), .nCS(nCS), .MISO(MISO)
		, .SPI_DATA_I(SPI_DATA_I[7:0])
		, .SPI_DATA_I_ACK(SPI_DATA_I_ACK)
		, .SPI_DATA_I_RDY(SPI_DATA_I_RDY)

		, .SPI_DATA(SEND_DATA[7:0])
		, .SPI_DATA_RDY(SPI_DATA_RDY)
		, .SPI_DATA_ACK(SPI_DATA_ACK)
	);

    assign SPI_DATA_I[7:0] =  (RS & r_recv_mode ) ? RECV_DATA[7:0]
						   : CTRL_REG[7:0] ;

    reg dummy_i_rdy;
    reg dummy_o_ack;
    always @(posedge CLK)
    begin
	r_recv_mode1 <= i_recv_mode;
    	dummy_o_ack <= SPI_DATA_RDY;
    	dummy_i_rdy <= ~SPI_DATA_I_ACK;
	if (~RS & SPI_DATA_RDY & SEND_DATA[7])
	  begin
	    r_recv_mode  <= SEND_DATA[1];
	    r_start      <= SEND_DATA[0];
	  end
	else
	  begin
	    if (r_se0)
	      begin
`ifdef AUTO_RECV_AFTER_SEND
		if (~r_recv_mode) r_recv_mode <= 1;
`endif
		r_start <= 1'b0;
	      end
	  end
    end
    assign RECV_DATA_ACK = RS & SPI_DATA_I_ACK;
    //assign SPI_DATA_I_RDY = RS ? RECV_DATA_RDY : dummy_i_rdy;
    assign SPI_DATA_I_RDY = dummy_i_rdy;
    assign SEND_DATA_RDY = RS & SPI_DATA_RDY;
    assign SPI_DATA_ACK = RS & ~i_recv_mode ? SEND_DATA_ACK : dummy_o_ack;

`elsif USE_BUS
    assign DB[7:0] =  (~nRD & ~nCS &  RS) ? RECV_DATA[7:0]
		    : (~nRD & ~nCS & ~RS) ? CTRL_REG[7:0]
		    : 8'bz;
    assign SEND_DATA[7:0] = DB[7:0];

    always @(posedge CLK)
    begin
	r_recv_mode1 <= i_recv_mode;
	if (~RS & ~(nWR | nCS))
	  begin
	    r_recv_mode <= SEND_DATA[1];
	    r_start     <= SEND_DATA[0];
	  end
	else
	  begin
	    if (r_se0)
	      begin
`ifdef AUTO_RECV_AFTER_SEND
		if (~r_recv_mode) r_recv_mode <= 1'b1;
`endif
	        r_start <= 1'b0;
	      end
	  end
    end
    assign RECV_DATA_ACK = ~(nRD | nCS);
    assign SEND_DATA_RDY = ~(nWR | nCS) & RS;
//
// BUS Hand shake 
//
//       _  ____________________________________  _____
// RS    _><____________________________________><_____
//
//       _____                                  _______
// nCS        |________________________________|
//
//        ______________               _____________
// nWR/nRD              |_____________|
//  
// WRITE:                    _________________
// DB[7:0]  ----------------<_________________>--------
//                      |---| CLK_4X (MAX)
//                      |---------------------| CLK_4X x 2 (MIN)
//                         
// READ:                   _____________
// DB[7:0]  --------------<_____________>----------
//  
//                      |-| gate level delay
`endif

    always @(negedge CLK)
    begin
	r_sync_req1 <= r_sync_req;
	r_recv_en1 <= r_recv_en;
	if (r_sync_req & ~RECV_EN | SE0) r_sync_req <= 1'b0;
	else if (i_recv_start & SEND_DONE) r_sync_req <= 1'b1;

	if (~i_recv_start | SE0) r_recv_en <= 1'b0;
	else if (SYNC_ACK) r_recv_en <= 1'b1;

	if (i_send_start & ((FS != 1)|S_DATA_RDY) & ~RECV_EN) r_send_en <= 1'b1;
	else if (SEND_DONE) r_send_en <= 1'b0;
    end

`ifdef USE_CRC
    assign crc5_in        = SEND_EN ? s_crc_out : crc_in;
    assign crc5_sel[1:0]  = SEND_EN ? s_crc5_sel[1:0] : r_crc5_sel[1:0];
    assign crc16_in       = SEND_EN ? s_crc_out : crc_in;
    assign crc16_sel[1:0] = SEND_EN ? s_crc16_sel[1:0] : r_crc16_sel[1:0];

    assign nCRC_ERROR = crc_match_sel & CRC16_MATCH | ~crc_match_sel & CRC5_MATCH | RECV_EN;
    //assign nCRC_ERROR = crc_match_sel ? CRC16_MATCH : CRC5_MATCH ;

    assign s_crc5_in  = crc5_out;
    assign s_crc16_in = crc16_out;

`endif
    assign I_DATA[7:0] = ~i_recv_mode ? SEND_DATA[7:0] : R_DATA[7:0];
    assign I_DATA_RDY = ~i_recv_mode ? SEND_DATA_RDY : R_DATA_RDY & RECV_EN;
    assign SEND_DATA_ACK = I_DATA_ACK;
    assign R_DATA_ACK = I_DATA_ACK;

    assign RECV_DATA[7:0] = O_DATA[7:0];
    assign S_DATA[7:0] = O_DATA[7:0];
    assign S_DATA_RDY = O_DATA_RDY & ~i_recv_mode;
    assign O_DATA_ACK = ~i_recv_mode ? S_DATA_ACK : RECV_DATA_ACK;

    assign nDATA_RDY = ~(O_DATA_RDY & i_recv_mode);
    assign nDATA_FULL = ~I_DATA_FULL;
    assign nDATA_EMPTY = ~I_DATA_EMPTY;
endmodule
`endif

`ifdef USE_CLKGEN
//                           _______
// CLK              |_______|
//
// clk_ph         3 | 0 | 1 | 2 | 3 | 0
//                  _____________
//  DP            ><             ><
//
//                  _________
// SYNC_ACK      __|
//
module udrv_clkgen # (
		parameter PH=0
	) (
                input CLK_4X
                , input DP
                , input SYNC_REQ
                , output SYNC_ACK
                , output CLK
	);
    reg [1:0] clk_ph = 0;
    reg r_dp = 1'b0;
    reg r_done = 1'b0;

    assign CLK = clk_ph[1];
    assign SYNC_ACK = r_done;

    always @(posedge CLK_4X)
    begin
	r_dp <= DP;
	if (~SYNC_REQ |  r_done | ~(r_dp ^ DP) | (clk_ph == PH) )
	    clk_ph <= clk_ph + 1;
	if (~SYNC_REQ)
		r_done <= 1'b0;
	else if ((r_dp ^ DP) & (clk_ph == PH))
		r_done <= 1'b1;
    end

endmodule
`endif

`ifdef USE_RECIEVER
module udrv_reciever (
                input CLK
                , input RECV_EN
                , input DP
                , input SE0
                , output [7:0] RECV_DATA
                , output RECV_DATA_RDY
                , input RECV_DATA_ACK
`ifdef USE_CRC
		, output O
		, output [1:0] SEL5
		, output [1:0] SEL16
		, output MATCH_SEL
`endif
	);

    reg recv_stat = 0;
    reg r_dp = 0;
    reg [7:0] data_out;
    reg data_set = 0;
    reg data_rdy = 0;
    reg data_ack = 0;
    reg [7:0] r_data;
    reg [1:0] data_bytes = 0;
//    reg [2:0] data_bytes = 0;
    reg [2:0] data_count = 0;
    reg [2:0] one_count = 0;
    reg [1:0] crc5_sel = 0;
    reg [1:0] crc16_sel = 0;
    reg r_match_sel = 1'b0;

    assign RECV_DATA[7:0] = data_out[7:0];
    assign RECV_DATA_RDY = data_rdy;

`ifdef USE_CRC
    reg r_o;
    assign O = r_o;
    assign SEL5[1:0] = crc5_sel[1:0];
    assign SEL16[1:0] = crc16_sel[1:0];
    assign MATCH_SEL = r_match_sel;
`endif
    always @(posedge CLK)
    begin
	data_ack <= RECV_DATA_ACK;
	r_dp <= DP;
`ifdef USE_CRC
	r_o <= ~(r_dp ^ DP);
`endif
	if (~RECV_EN | SE0) 
	  begin
	    recv_stat <= 0;
	    data_count <= 0;
	    r_data <= 0;
	    one_count <= 0;
	    crc5_sel <= 2'b00;
	    crc16_sel <= 2'b00;
	  end
	else if (recv_stat == 0) 
	  begin
	    if (~(r_dp ^ DP)) // end of SYNC
	      begin
	    	recv_stat <= 1;
	        crc5_sel <= 2'b11;
	        crc16_sel <= 2'b11;
	        data_bytes <= 0;
		r_match_sel <= 1'b0;
	      end
	  end
	else
	  begin
	    if (~(r_dp ^ DP)) one_count <= one_count + 1;
	    else              one_count <= 0;

	    if (one_count <= 5)
	      begin
		r_data <= { ~(r_dp ^ DP), r_data[7:1] };
		data_count <= data_count + 1;

		if (data_count == 7)
		    begin
			data_out <= { ~(r_dp ^ DP), r_data[7:1] };
	        	if (&data_bytes == 1'b0) data_bytes <= data_bytes + 1;
			// use CRC-16 if DATAx packet
    			if ( (data_bytes == 0) & (r_data[2:1] == 2'b11) )
				r_match_sel <= 1'b1;
		    end
	      end

	    if ((one_count > 5) | (data_bytes < 1))
	      begin
	    	crc5_sel  <= 2'b00; // NOP
	    	crc16_sel <= 2'b00; // NOP
	      end
	    else
	      begin
	    	crc5_sel  <= ( (data_bytes == 2) & (data_count >= 3) )
			     ? 2'b10 // Extract
			     : 2'b01 // Calc
			     ;
	    	crc16_sel <= 2'b01; // Calc
	      end
	  end

	data_set <= ( RECV_EN & ~SE0 & (recv_stat == 1) 
		    & (one_count <= 5) & (data_count == 7) );

	if (data_set)
		data_rdy <= 1'b1;
	else if (~data_ack & RECV_DATA_ACK)
		data_rdy <= 1'b0;
    end
endmodule
`endif

`ifdef USE_SENDER
//  send_stat        ... 0  0  1  2  2 ...   2  2  3  4  5  0  0 ...
//  DP,DM            01         ><DDDDD ... DDDDD>< 00 ><   01 ....
//
//  SEND_DATA_ACK    ____________________|______________
//                 ____________                              _____
//  SEND_DONE                  |____________________________|
//
//
module udrv_sender # (
		parameter FS=1
		, parameter SUPPORT_CRC5S=1
	) (
                input CLK
                , input SEND_EN
                , output SEND_DONE
                , output DP
                , output DM
                , output OE
                , input [7:0] SEND_DATA
                , input SEND_DATA_RDY
                , output SEND_DATA_ACK
`ifdef USE_CRC
		, input I5
		, input I16
		, output O
		, output [1:0] SEL5
		, output [1:0] SEL16
`endif
	);

    reg r_dp;
    reg r_dm;
    reg [2:0] send_stat = 0;
    reg r_out = 1'b0;
    reg r_oe = 1'b0;
    reg [6:0] data_in;
    reg data_ack = 0;
    reg [3:0] data_count = 0;
    reg [2:0] one_count = 0;
    reg [2:0] data_bytes = 0;
    reg use_crc16 = 1'b0;
    reg crc16_extract = 0;
    reg [1:0] crc5_sel = 0;
    reg [1:0] crc16_sel = 0;
    reg crc_out = 1'b0;

    assign DP = r_dp;
    assign DM = r_dm;
    assign OE = r_oe;
    assign SEND_DONE = (send_stat == 0);
    assign SEND_DATA_ACK = data_ack;

`ifdef USE_CRC
    assign O = crc_out;
    assign SEL5[1:0]  = crc5_sel[1:0];
    assign SEL16[1:0] = crc16_sel[1:0];
    wire OUT_BIT = (crc16_extract ? I16 : crc5_extract ? I5 : r_out);
`else
    wire OUT_BIT = r_out;
    wire I5 = r_out;
`endif

    wire crc5_extract = SUPPORT_CRC5S & ~use_crc16 
			& (data_bytes == 3) & (data_count < 5);

    always @(posedge CLK)
    begin
        r_oe <= (send_stat != 0);
        crc_out <= crc5_extract ? I5 : r_out;
	crc5_sel <= (send_stat == 1) ? 2'b11 // RESET
		: (one_count > 5)  ? 2'b00 // NOTHING
		: ( crc5_extract )     ? 2'b10 // Extract
		: ( data_bytes >= 2 ) ? 2'b01 // Calc.
		: 2'b00;

	crc16_sel <= (send_stat == 1) ? 2'b11 // RESET
		: (one_count > 5)  ? 2'b00 // NOTHING
		: ( crc16_extract )     ? 2'b10 // Extract
		: ( data_bytes >= 2 ) ? 2'b01 // Calc.
		: 2'b00;

	if (~SEND_EN)
	  begin
	    send_stat <= 0;
	    data_count <= 7;
	  end
	if ( (FS != 1) & (send_stat == 0) & SEND_EN  & ~SEND_DATA_RDY)
	    send_stat <= 3; // EOP only
	else
	if ( (send_stat == 0) & SEND_EN )
	  begin
	    send_stat <= 1;
	    data_in[6:0] <= 7'b1000000;
	    r_out <= 1'b0;
	    data_bytes <= 0;
	    crc16_extract <= 1'b0;
	    use_crc16 <=  (SEND_DATA[1] & SEND_DATA[0]); // PID == xxxxxx11
	  end
	else if (send_stat == 1)
	    send_stat <= 2;
	else if ( (send_stat == 2) & (one_count <= 5) & (data_count == 0) )
	    if (SEND_DATA_RDY)
	      begin
	        if (&data_bytes == 1'b0) data_bytes <= data_bytes + 1;
		data_count <= 7;
		data_in[6:0] <= SEND_DATA[7:1];
		r_out <= SEND_DATA[0];
	      end
`ifdef USE_CRC
	    else if (use_crc16 & ~crc16_extract)
	      begin
		data_count <= 15;
		crc16_extract <= 1'b1;
	      end
`endif
	    else // exit sending
		    send_stat <= 3;
	else if ( (send_stat == 2) & (one_count <= 5) )
	  begin
		data_count <= data_count - 1;
		data_in[6:0] <= { 1'b0, data_in[6:1] };
		r_out <= data_in[0];
	  end
	else if (send_stat == 3) send_stat <= 4;
	else if (send_stat == 4) send_stat <= 5;
	else if (send_stat == 5) send_stat <= 0;

	if ( (send_stat == 3) | (send_stat == 4) ) // SE0
	  begin
	    one_count <= 0;
	    r_dp <= 1'b0;
	    r_dm <= 1'b0;
	  end
	else if (send_stat == 2) 
	  begin
	    if ( ~(OUT_BIT) | (one_count > 5) )
	      begin
		one_count <= 0;
		r_dp <= ~r_dp;
		r_dm <= r_dp;
	      end
	    else one_count <= one_count + 1;
	  end
	else // if ( (send_stat == 1) | (send_stat == 5) )
	  begin
	    one_count <= 0;
	    r_dp <= (FS) ? 1'b1 : 1'b0;
	    r_dm <= (FS) ? 1'b0 : 1'b1;
	  end

	if (SEND_EN & SEND_DATA_RDY & ( (send_stat == 2)  
		      & (one_count <= 5) & (data_count == 0) ) )
	    data_ack <= 1'b1;
	else if (~SEND_DATA_RDY)
	    data_ack <= 1'b0;
    end
endmodule
`endif

`ifdef USE_FIFO
//
// Hand shake 
//         ______________________  _______
// DATA   <______________________><_______
//
//                _______                _____
// RDY   ________|       |______________|
//  
//                    ________
// ACK  _____________|       |________________
//  
//
// CLK   |       |       |       |       |  
//  
module udrv_fifo # (
//  		parameter DEPTH = 5
		parameter DEPTH = 6
		, parameter DELAYED_INPUT = 0
	) (
                input CLK
                , input RST
                , input [7:0] I_DATA
                , input I_DATA_RDY
                , output I_DATA_ACK
                , output I_DATA_FULL
                , output I_DATA_EMPTY
                , output [7:0] O_DATA
                , output O_DATA_RDY
                , input O_DATA_ACK
	);
    reg  [7:0] mem [0:2**DEPTH-1];

    reg [DEPTH-1:0] i_addr = 0;
    reg [DEPTH-1:0] o_addr = 0;
    wire [DEPTH-1:0] i_next = i_addr + 1;
    wire [DEPTH-1:0] o_next = o_addr + 1;
    reg i_rdy = 0;
    reg i_ack = 0;
    reg o_rdy = 0;
    reg o_ack = 0;
    reg r_full = 0;
    assign I_DATA_FULL = r_full;
    assign I_DATA_EMPTY = (i_addr == o_addr);
    assign I_DATA_ACK = i_ack;
    assign O_DATA_RDY = o_rdy;
    assign DATA_RDY = DELAYED_INPUT ? i_rdy : I_DATA_RDY;

    always @(negedge CLK)
    begin
        i_rdy <= I_DATA_RDY;
        o_ack <= O_DATA_ACK;
        if (RST)
	  begin
	    i_addr <= 0;
	    o_addr <= 0;
            i_ack <= 1'b0;
            o_rdy <= 1'b0;
            r_full <= 1'b0;
	  end
        else if (DATA_RDY & ~i_ack) 
	  begin
	    if (i_next == o_addr)
	      r_full <= 1'b1;
	    else
	      begin
		mem[i_addr] <= I_DATA;
		i_addr <= i_next;
	      end
            i_ack <= 1'b1;
	  end
	else
	    i_ack <= DATA_RDY;

        if ( o_ack & ~O_DATA_ACK & (i_addr != o_addr))
	  begin
	    o_addr <= o_next;
            o_rdy <= 1'b0;
	  end
        else if ( O_DATA_ACK )
            o_rdy <= 1'b0;
        else
	    o_rdy <= (i_addr != o_addr);
    end
    assign O_DATA[7:0] = mem[o_addr];

endmodule
`endif

`ifdef USE_CRC
// CRC5 memo :
//  IN 09 CRC 13 PKT  80 A5 09 98
//  IN 0F CRC 03 PKT  80 A5 0F 18
module udrv_crc # (
		parameter WIDTH=16
		// parameter WIDTH=5
		, parameter SEQUENTIAL_MATCHING=0
	) (
		input CLK
		, input [1:0] SEL
		, input I	// LSB first
		, output O	// LSB first
                , output MATCH
		, output [WIDTH-1:0] CRC
		);

    reg [WIDTH-1:0] r_crc = (-1);
    wire I2 = (r_crc[0] ^ I);
    assign CRC [WIDTH-1:0] = ~r_crc[WIDTH-1:0] ;
    assign O = ~r_crc[0];
    reg r_match = 0;
    assign MATCH = (SEQUENTIAL_MATCHING) ? r_match
			: (WIDTH == 5) ? ( ~r_crc == 5'b11001 )
			: (WIDTH == 16) ? ( ~r_crc == 16'h4FFE )
			: 0;

    always @(negedge CLK)
    begin
	if ( SEL == 2'b11 ) // RST
	    begin
		r_crc <= (-1);
    		r_match <= 1;
	    end
	else if ( SEL == 2'b10 ) // Extract bitstream
	    begin
		r_match <= (I ^ r_crc[0]) & r_match;
		r_crc <= { 1'b0, r_crc[WIDTH-1:1] };
	    end
	else if ( SEL == 2'b01 ) // Calc.
	    if (WIDTH == 16)
		r_crc <= { I2 ,  r_crc[15]
			, ( r_crc[14] ^ I2) ,  r_crc[13:2]
			, ( r_crc[ 1] ^ I2) } ;
	    else if (WIDTH == 5)
		r_crc <= { I2, r_crc[4]
			, ( r_crc[ 3] ^ I2), r_crc[2:1] };
    end
endmodule
`endif

`ifdef USE_SPI
module udrv_spi (
		  input MOSI
		, input SCK
		, input nCS
		, output MISO

		, input [7:0] SPI_DATA_I
		, output SPI_DATA_I_ACK
		, input SPI_DATA_I_RDY

		, output [7:0] SPI_DATA
		, output SPI_DATA_RDY
		, input SPI_DATA_ACK
	);

    reg [2:0]  r_count = 0;
    reg [7:0]  r_spi = 0;
    reg [7:0]  spi_out = 0;
    reg r_mosi = 1'b0;

    reg o_data_ack  = 1'b0;
    reg o_data_rdy  = 1'b0;
    assign SPI_DATA_RDY   = o_data_rdy;

    reg i_data_ack  = 1'b0;
    assign SPI_DATA_I_ACK = i_data_ack;    
     
    assign SPI_DATA       = spi_out[7:0];
    assign MISO = (r_count == 0) ? SPI_DATA_I[7] : r_spi[7];

    always @(posedge SCK)
    begin
	r_mosi <= MOSI;
	if (r_count == 1)
	    i_data_ack <= 1'b1;
	else if (~SPI_DATA_I_RDY)
     	    i_data_ack <= 1'b0;
    end

    always @(posedge nCS or negedge SCK)
    begin
	if (nCS) //  LOAD
	  begin
	    r_count <= 0;
	    r_spi <= 0;
	    o_data_rdy <= 1'b0;
	    o_data_ack  <= 1'b0;
	  end
	else
	  begin
	    r_count <= r_count + 1;
	    o_data_ack  <=  SPI_DATA_ACK;
	    if (r_count == 0)
		r_spi <=  { SPI_DATA_I[6:0], r_mosi };	//  LOAD
	    else
	        r_spi <= {r_spi[6:0], r_mosi };		// SHIFT

	    if (r_count == 7)
		spi_out <= {r_spi[6:0], r_mosi };

	    if (r_count == 7)
	        o_data_rdy <= 1'b1;
	    else if (~o_data_ack & SPI_DATA_ACK)
	        o_data_rdy <= 1'b0;
	  end
    end
endmodule
`endif
