/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 //      original 2KB RAM
//  A7  1            24  VCC  
//  A6  2            23  A8 
//  A5  3            22  A9  
//  A4  4            21  /WE
//  A3  5            20  /OE 
//  A2  6            19  A10
//  A1  7            18  /CE
//  A0  8            17  D7
//  D0  9            16  D6
//  D1  10           15  D5
//  D2  11           14  D4
// GND  12           13  D3 
//
//  XO2-256    modified ROM/RAM    XO2-256
//  27   A7  1            24  VCC  
//  28   A6  2         x  23  A8    25 (N.C.)
//  29   A5  3         x  22  A9    23 (N.C.)
//  30   A4  4            21  /WE   21
//  32   A3  5            20  /OE   20
//   1   A2  6          x 19  D7    17
//   4   A1  7            18  /CE   16
//   5   A0  8          x 17  N.C.
//   8   D0  9            16  D6    14
//   9   D1  10           15  D5    13
//  10   D2  11           14  D4    12
//      GND  12           13  D3    11

//`define RAM_USE_CLK
// `define RAM_INCLUDE_DAT
 
module sram128 (
                input WE,
		input OE,
		input CE,
                input [7:0] A,
                inout [7:0] D
);
	wire [7:0] DIB;
	wire [7:0] DOB;

	sram #(.SIZE(128)) sram_inpl (.CLK(CE), .WEB(WE) , .ADDRB(A[7:0]), .DIB(DIB[7:0]), .DOB(DOB[7:0]));

	assign D[7:0] = (!CE & !OE & WE) ? DOB[7:0] : 8'bz;  
	assign DIB[7:0] = D[7:0];
endmodule

module sram # (
                parameter SIZE = 128
//		parameter SIZE = 256
) (
                input CLK,
		input WEB,
                input [7:0] ADDRB,
                input [7:0] DIB,
                output [7:0] DOB
);
    reg  [7:0] mem [0:SIZE-1];
`ifdef RAM_USE_CLK
    reg [7:0] r_addr;
    always @(negedge CLK)
    begin
        if (!WEB) mem[ADDRB] <= DIB;
	r_addr <= ADDRB;
    end
    assign DOB = mem[r_addr];
`else
    always @(posedge WEB)
    begin
        mem[ADDRB] <= DIB;
    end
    assign DOB = mem[ADDRB];
`endif

`ifdef RAM_INCLUDE_DAT
  initial
    begin
       $readmemh("rom_data.mem" , mem, 0 , SIZE-1);
    end
`endif

endmodule
