`timescale 10ps / 10ps
`define GTK // use VCD

`define MODULE	tb_udrv
`ifdef CLK_LIMIT
`else
`define CLK_LIMIT	1500
`endif

`define USE_SPI
`define USE_BUS
`define LB_TEST

module `MODULE;

   reg [23:0] CLK_COUNT;
   reg CLK;   // input
   reg RESET; // input
   reg [7:0] data;
   reg RS   = 1'b0;
   reg nCS   = 1'b1;
   reg SCK  = 1'b0;
   reg MOSI = 1'b0;

   wire DP;
   wire DM;
`ifdef USE_CRC
//   wire SEND_CRC5 = 1'b1;
`endif

`ifdef LB_TEST
   reg RS2   = 1'b1;
   reg nCS2   = 1'b1;
   reg CS2LCK = 1'b0;
   reg SCK2  = 1'b0;
   reg MOSI2 = 1'b0;
`endif

udrv #( .FS(1) , .FIFO_DEPTH(3) ) top ( .CLK_4X(CLK) 
`ifdef LB_TEST
		, .DPR(DPR)
		, .DMR(DMR)
		, .DPW(DPW)
		, .DMW(DMW)
`else
		, .DP(DP)
		, .DM(DM)
`endif
		, .nDATA_RDY(nRECV_DATA_RDY)
		, .nIRQ(nIRQ)
`ifdef USE_SPI
		, .SCK(SCK)
		, .nCS(nCS)
		, .MOSI(MOSI)
		, .MISO(MISO)
		, .RS(RS)
`elsif USE_BUS
		, .DB(DB)
		, .nCS(nCS)
		, .RS(RS)
		, .nRD(nRD)
		, .nWR(nWR)
`else
		, .RECV_DATA_ACK(RECV_DATA_ACK)
		, .SEND_DATA_RDY(SEND_DATA_RDY)
		, .SEND_DATA_ACK(SEND_DATA_ACK)
`endif
`ifdef USE_CRC
//		, .SEND_CRC5(SEND_CRC5)
`endif
		);

`ifdef LB_TEST
udrv #( .FS(1) , .FIFO_DEPTH(3) ) top2 ( .CLK_4X(CLK) 

		, .DPR(DPR2)
		, .DMR(DMR2)
		, .DPW(DPW2)
		, .DMW(DMW2)

		, .nDATA_RDY(nRECV_DATA_RDY2)
		, .nIRQ(nIRQ2)
`ifdef USE_SPI
		, .SCK(SCK2)
		, .nCS(nCS2)
		, .MOSI(MOSI2)
		, .MISO(MISO2)
		, .RS(RS2)
`elsif USE_BUS
		, .DB(DB2)
		, .nCS(nCS2)
		, .RS(RS2)
		, .nRD(nRD2)
		, .nWR(nWR2)
`else
		, .RECV_DATA_ACK(RECV_DATA_ACK2)
		, .SEND_DATA_RDY(SEND_DATA_RDY2)
		, .SEND_DATA_ACK(SEND_DATA_ACK2)
`endif
		);

`ifdef USE_SPI
   reg [3:0] CLK_SCK = 0;
   reg [3:0] BIT_CNT = 0;
   always @(posedge CLK)
   if (~CS2LCK) begin
	if (nCS2 & ~nRECV_DATA_RDY2)
	  begin
	    nCS2 <= 1'b0;
	    CLK_SCK <= 0;
	    BIT_CNT <= 0;
	  end
	else CLK_SCK <= CLK_SCK + 1;

	if (~nCS2 & (CLK_SCK == 2))
	  begin
	         if (~nRECV_DATA_RDY2 & BIT_CNT == 8)
	  	    begin
	    		BIT_CNT <= 1;
	  	    end
	         if (nRECV_DATA_RDY2 & BIT_CNT == 8) nCS2  <= 1'b1;
		 else
	  	    begin
	                SCK2 <= 1'b1;
	                BIT_CNT <= BIT_CNT + 1;
	  	    end
	  end
	else if (CLK_SCK == 4)
	  begin
		 SCK2 <= 1'b0;
	    	 CLK_SCK <= 0;
	  end
   end
`elsif USE_BUS
   reg [4:0] CLK_SCK = 0;
   always @(posedge CLK)
   if (~CS2LCK) begin
	if (nCS2 & ~nRECV_DATA_RDY2)
	  begin
	    nCS2 <= 1'b0;
	    CLK_SCK <= 0;
	  end
	else CLK_SCK <= CLK_SCK + 1;

	if (~nCS2 & (CLK_SCK == 1))
	  begin
	    nRD2 <= 1'b0;
	  end
	else if (~nCS2 & (CLK_SCK == 7))
	  begin
	    nRD2 <= 1'b1;
	  end
	else if (~nCS2 & (CLK_SCK == 8))
	  begin
	    nCS2 <= 1'b1;
	  end
   end
`endif
   assign DPR2 = DPW;
   assign DMR2 = DMW;

   assign DPR = DPW2;
   assign DMR = DMW2;
`endif

   parameter CLKP = 5000000;        //  50us
   parameter CLKH = CLKP/2;      //
   parameter CLKL = CLKP-CLKH;   //
   
   always begin
      #CLKH CLK = 0;
      #CLKL CLK = 1;
   end

  integer fp;
  initial begin
    fp = $fopen("tb.log");
  end

`ifdef GTK
   initial begin
      $dumpfile("tb.vcd");
      $dumpvars(0,`MODULE.top);
`ifdef LB_TEST
      $dumpvars(0,`MODULE.top2);
`endif
   end
`endif

   initial begin : test
      CLK_COUNT = 0;
      //SEND_MODE = 1'b0;
      //SEND_DATA_RDY = 1'b0;
      sys_reset();
      sys_wait(40);
   nCS2 = 1'b0; CS2LCK = 1'b1; sys_wait(5);
   io_out2(12'h183); // RECV_START
   sys_wait(5); nCS2 = 1'b1; CS2LCK = 1'b0;
   nCS = 1'b0; sys_wait(5);
   io_out(12'h180); // SEND_MODE
   sys_wait(5); nCS = 1'b1;
      sys_wait(5);
      nCS = 1'b0; sys_wait(5);
      io_out(12'h0A5);
      io_out(12'h009);
      io_out(12'h000);
      sys_wait(5); nCS = 1'b1;
      //SEND_START = 1'b1;
      sys_wait(5);
   nCS = 1'b0; sys_wait(5);
   io_out(12'h181); // SEND_START
   sys_wait(5); nCS = 1'b1;
      sys_wait(150);
   nCS2 = 1'b0; CS2LCK = 1'b1; sys_wait(5);
   io_out2(12'h183); // RECV_START
   sys_wait(5); nCS2 = 1'b1; CS2LCK = 1'b0;
   nCS = 1'b0; sys_wait(5);
   io_out(12'h180); // SEND_MODE
   sys_wait(5); nCS = 1'b1;
      sys_wait(50);
      //SEND_START = 1'b0;
      nCS = 1'b0; sys_wait(5);
      io_out(12'h0C3);
      io_out(12'h009);
      io_out(12'h098);
      sys_wait(5); nCS = 1'b1;
      sys_wait(10);
      nCS = 1'b0; sys_wait(5);
      io_out(12'h0F9);
      io_out(12'h0B5);
      sys_wait(5); nCS = 1'b1;
      //sys_wait(200);
      //SEND_START = 1'b1;
   nCS2 = 1'b0; CS2LCK = 1'b1; sys_wait(5);
   io_out2(12'h183); // RECV_START
   sys_wait(5); nCS2 = 1'b1; CS2LCK = 1'b0;
   nCS = 1'b0; sys_wait(5);
   io_out(12'h181); // SEND_START
   sys_wait(5); nCS = 1'b1;
CS2LCK = 1'b1;
      sys_wait(200);
CS2LCK = 1'b0;
      
      while ( (CLK_COUNT < `CLK_LIMIT) )
      begin
         @(posedge CLK);
         CLK_COUNT = CLK_COUNT + 1;
      end
//      @(posedge CLK);
      #1
      $display("*** Simulation Complete! ***");
      $finish;
   end

`ifdef USE_SPI
   task io_out;
      input [11:0] i_data;
      reg [7:0] i;
      begin
        RS = ~i_data[8];
        i = 0;
        while ( i < 8 )
        begin
	   MOSI = i_data[7];
	   sys_wait(2);
	   SCK = 1'b1;
	   sys_wait(2);
	   SCK = 1'b0;
	   i_data[7:0] = { i_data[6:0] , MISO };
           i = i + 1;
        end
	//sys_wait(5);
        //RS = 1'b0;
      end
   endtask // io_out
`elsif USE_BUS
   task io_out;
      input [12:0] i_data;
      begin
         RS = ~i_data[8];
	 sys_wait(2);
         data = i_data[7:0];
	 sys_wait(5);
         nWR = 1'b0;
	 sys_wait(5);
         nWR = 1'b1;
	 sys_wait(2);
         //RS = 1'b0;
      end
   endtask // io_out
`endif

`ifdef USE_SPI
   task io_out2;
      input [11:0] i_data;
      reg [7:0] i;
      begin
        RS2 = ~i_data[8];
        i = 0;
        while ( i < 8 )
        begin
	   MOSI2 = i_data[7];
	   sys_wait(2);
	   SCK2 = 1'b1;
	   sys_wait(2);
	   SCK2 = 1'b0;
	   i_data[7:0] = { i_data[6:0] , MISO };
           i = i + 1;
        end
	sys_wait(5);
        RS2 = 1'b1;
      end
   endtask // io_out2
`elsif USE_BUS
   task io_out2;
      input [12:0] i_data;
      begin
         RS2 = ~i_data[8];
	 sys_wait(2);
         data = i_data[7:0];
	 sys_wait(5);
         nWR2 = 1'b0;
	 sys_wait(5);
         nWR2 = 1'b1;
	 sys_wait(2);
         RS2 = 1'b1;
      end
   endtask // io_out
`endif

   task sys_wait;
      input [7:0] count;
      reg [7:0] c;
      begin
        c = 0;
        while ( c < count )
        begin
           @(posedge CLK);
           CLK_COUNT = CLK_COUNT + 1;
           c = c + 1;
        end
      end
   endtask // sys_wait

   task sys_reset;
      begin
         #1 RESET = 0;
         @(posedge CLK);
         @(posedge CLK);
         #1 RESET = 1;
         @(posedge CLK);
         @(posedge CLK);
         #1 RESET = 0;
         @(posedge CLK);
         @(posedge CLK);
         CLK_COUNT = CLK_COUNT + 1;
      end
   endtask // sys_reset
   
endmodule // test
